#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# dashas.py -- module for Dasha computation. All Dahsa data computed for given birth details handled here.
#   Dashas [Vimshottari etc]
# 
# Copyright (C) 2022 Shyam Bhat  <vicharavandana@gmail.com>
# Downloaded from "https://github.com/VicharaVandana/jyotishyam.git"
#
# This file is part of the "jyotishyam" Python library
# for computing Hindu jataka with sidereal lahiri ayanamsha technique 
# using swiss ephemeries
#

#import necessary modules
from datetime import datetime, timedelta

from dateutil.relativedelta import relativedelta

import parashara.mod_astrodata as data
from parashara.mod_general import *

dashaVimshottariSkeleton = {"Venus":{   "Nakshatras": ["Bharani", "Purva Phalguni", "Purva Ashadha"],
                                        "duration"  : 20, #20 years out of 120 years
                                        "percentage": (20.0/120),
                                        "prev-dasha": "Ketu",
                                        "next-dasha": "Sun"
                                    },
                            "Sun":  {   "Nakshatras": ["Kritika", "Uttara Phalguni", "Uttara Ashadha"],
                                        "duration"  : 6, #6 years out of 120 years
                                        "percentage": (6.0/120),
                                        "prev-dasha": "Venus",
                                        "next-dasha": "Moon"
                                    },
                            "Moon": {   "Nakshatras": ["Rohini", "Hasta", "Shravana"],
                                        "duration"  : 10, #10 years out of 120 years
                                        "percentage": (10.0/120),
                                        "prev-dasha": "Sun",
                                        "next-dasha": "Mars"
                                    },
                            "Mars": {   "Nakshatras": ["Mrigashira", "Chitra", "Dhanishta"],
                                        "duration"  : 7, #7 years out of 120 years
                                        "percentage": (7.0/120),
                                        "prev-dasha": "Moon",
                                        "next-dasha": "Rahu"
                                    },
                            "Rahu": {   "Nakshatras": ["Ardra", "Swati", "Shatabhishak"],
                                        "duration"  : 18, #18 years out of 120 years
                                        "percentage": (18.0/120),
                                        "prev-dasha": "Mars",
                                        "next-dasha": "Jupiter"
                                    },
                            "Jupiter": {"Nakshatras": ["Punarvasu", "Vishaka", "Purva Ashadha"],
                                        "duration"  : 16, #16 years out of 120 years
                                        "percentage": (16.0/120),
                                        "prev-dasha": "Rahu",
                                        "next-dasha": "Saturn"
                                    },
                            "Saturn": { "Nakshatras": ["Pushya", "Anurada", "Uttara Bhadrapada"],
                                        "duration"  : 19, #19 years out of 120 years
                                        "percentage": (19.0/120),
                                        "prev-dasha": "Jupiter",
                                        "next-dasha": "Mercury"
                                      },
                            "Mercury": {"Nakshatras": ["Ashlesha", "Jyeshta", "Revati"],
                                        "duration"  : 17, #17 years out of 120 years
                                        "percentage": (17.0/120),
                                        "prev-dasha": "Saturn",
                                        "next-dasha": "Ketu"
                                    },
                            "Ketu": {   "Nakshatras": ["Ashwini", "Magha", "Mula"],
                                        "duration"  : 7, #7 years out of 120 years
                                        "percentage": (7.0/120),
                                        "prev-dasha": "Mercury",
                                        "next-dasha": "Venus"
                                    }
                            }

#Takes Moon longitude in seconds, Dasha lord and Birth date and gives the start date of Dasha lords dasha
def computeStartDate_FirstDashaLord(lngsecondsMoon, dashaLord, birthDate):
    #lenth of a nakshatra in seconds is 13deg x 3600 + 20min x 60
    l_nakLenseconds = ((13*3600)+(20*60))
    #dasha lord dasha duration in days
    l_dashaDurationDays = 365.25 * dashaVimshottariSkeleton[dashaLord]["duration"]
    #How much moon has progressed into the nakshatra in seconds 
    l_deltalngseconds = lngsecondsMoon % l_nakLenseconds

    #calculate how many days have passed on birthday since dasha lord started his dasha
    l_elapsedDashaDurationDays = ((l_deltalngseconds * l_dashaDurationDays) / l_nakLenseconds)
    ##print(f'elapsed duration in days is {l_elapsedDashaDurationDays}')
    #Calculate the start date of dasha from subtracting elapsed date from birthday
    l_dashaStartDate = birthDate - timedelta(days=l_elapsedDashaDurationDays)

    return(l_dashaStartDate)

vimshottariDasha = []
mahadashaPlanetEntry = {
                        "name": "",
                        "startDate": datetime(year=1, month=1, day=1, hour=0, minute=0, second=0),
                        "endDate": datetime(year=1, month=1, day=1, hour=0, minute=0, second=0),
                        "entryString": "",
                        "level": "mahadasha",
                        "sublevel": "antardasha",
                        "antardasha" : []   #contains array of Antardasha planet entries
                        }
antardashaPlanetEntry = {
                        "name": "",
                        "startDate": datetime(year=1, month=1, day=1, hour=0, minute=0, second=0),
                        "endDate": datetime(year=1, month=1, day=1, hour=0, minute=0, second=0),
                        "entryString": "",
                        "level": "antardasha",
                        "sublevel": "paryantardasha",
                        "paryantardasha" : []   #contains array of paryantardasha planet entries
                        }
paryantardashaPlanetEntry = {
                        "name": "",
                        "startDate": datetime(year=1, month=1, day=1, hour=0, minute=0, second=0),
                        "endDate": datetime(year=1, month=1, day=1, hour=0, minute=0, second=0),
                        "entryString": "",
                        "level": "paryantardasha",
                        "sublevel": "sookshma-antardasha",
                        #"paryantardasha" : []   #contains array of paryantardasha planet entries
                        }
#dashaStrings = []
#dashaCodeLines = []

def clearDashaDetails():
    #global dashaStrings
    #global dashaCodeLines
    global vimshottariDasha
    global mahadashaPlanetEntry
    global antardashaPlanetEntry
    global paryantardashaPlanetEntry
    #dashaStrings = []
    #dashaCodeLines = []
    vimshottariDasha = []
    mahadashaPlanetEntry = {
                            "name": "",
                            "startDate": datetime(year=1, month=1, day=1, hour=0, minute=0, second=0),
                            "endDate": datetime(year=1, month=1, day=1, hour=0, minute=0, second=0),
                            "entryString": "",
                            "level": "mahadasha",
                            "sublevel": "antardasha",
                            "antardasha" : []   #contains array of Antardasha planet entries
                            }.copy()
    antardashaPlanetEntry = {
                            "name": "",
                            "startDate": datetime(year=1, month=1, day=1, hour=0, minute=0, second=0),
                            "endDate": datetime(year=1, month=1, day=1, hour=0, minute=0, second=0),
                            "entryString": "",
                            "level": "antardasha",
                            "sublevel": "paryantardasha",
                            "paryantardasha" : []   #contains array of paryantardasha planet entries
                            }.copy()
    paryantardashaPlanetEntry = {
                            "name": "",
                            "startDate": datetime(year=1, month=1, day=1, hour=0, minute=0, second=0),
                            "endDate": datetime(year=1, month=1, day=1, hour=0, minute=0, second=0),
                            "entryString": "",
                            "level": "paryantardasha",
                            "sublevel": "sookshma-antardasha",
                            #"paryantardasha" : []   #contains array of paryantardasha planet entries
                            }.copy()
    return(True)
def computeVimshottariDasha(lngsecondsMoon, nakshatraLord, birthDate):
    #global dashaStrings
    #global dashaCodeLines
    global vimshottariDasha
    global mahadashaPlanetEntry
    global antardashaPlanetEntry
    global paryantardashaPlanetEntry
    idcnt = 0
    l_DashaStartDate = computeStartDate_FirstDashaLord(lngsecondsMoon, nakshatraLord, birthDate)
    currentDate = datetime.now()
    data.charts["Dashas"]["Vimshottari"]["current"]["date"] = str(currentDate)
    data.charts["Dashas"]["Vimshottari"]["current"]["dasha"] = ""
    data.charts["Dashas"]["Vimshottari"]["current"]["bhukti"] = ""
    data.charts["Dashas"]["Vimshottari"]["current"]["paryantardasha"] = ""
    #print(f'start date is :{l_DashaStartDate}')
    l_firstPlanet = nakshatraLord
    l_wholeDuration = relativedelta(years=120)  #whole vimshottari dasha is for 120 years
    #compute Vimshottari table
    res_mahadasha = computeSubPeriods(l_DashaStartDate, l_firstPlanet, l_wholeDuration, "Mahadasha", birthDate)
    mahadasha_cnt = 0
    for item in res_mahadasha:
        mahadashaPlanetEntry["name"] = item["name"]
        mahadashaPlanetEntry["startDate"] = item["startDate"]
        mahadashaPlanetEntry["endDate"] = item["endDate"]
        mahadashaPlanetEntry["entryString"] = item["entryString"]
        #dashaStrings.append(f'Dasha: {item["entryString"]} iid={idcnt}, open=False')
        #dashaCodeLines.append(f'tree.insert("", END, text = "{item["entryString"]}", iid={idcnt}, open=False)')
        dashaIdx = idcnt
        antar_pos = 0
        idcnt = idcnt + 1
        #Adding Mahadasha details
        mahadasha_cnt = mahadasha_cnt + 1
        mahadashaPlanetName = mahadashaPlanetEntry["name"]
        l_mahadasha = {}
        l_mahadasha["lord"] = mahadashaPlanetName
        l_mahadasha["dashaNum"] = mahadasha_cnt
        l_mahadasha["startDate"] = str(item["startDate"])
        l_mahadasha["endDate"] = str(item["endDate"])
        startage = (item["startage"])
        endAge = (item["endage"])
        l_dur = (endAge-startage)      
        l_mahadasha["duration"] = f" {l_dur.years}yr {l_dur.months}m {l_dur.days}d"
        l_mahadasha["startage"] = f" {startage.years}yr {startage.months}m {startage.days}d"
        l_mahadasha["endage"] = f" {endAge.years}yr {endAge.months}m {endAge.days}d"  
        if(currentDate >= item["startDate"]) and (currentDate < item["endDate"]):
            data.charts["Dashas"]["Vimshottari"]["current"]["dasha"] = l_mahadasha.copy()["lord"]
        
        data.charts["Dashas"]["Vimshottari"]["mahadashas"][mahadashaPlanetName] = l_mahadasha.copy()

        #for Antardasha
        l_DashaStartDate = item["startDate"]
        l_firstPlanet = item["name"]
        l_wholeDuration = item["endDate"] - item["startDate"]
        res_antardasha = computeSubPeriods(l_DashaStartDate, l_firstPlanet, l_wholeDuration, "Antardasha", birthDate)
        antardasha_cnt = 0
        for item2 in res_antardasha:
            antardashaPlanetEntry["name"] = item2["name"]
            antardashaPlanetEntry["startDate"] = item2["startDate"]
            antardashaPlanetEntry["endDate"] = item2["endDate"]
            antardashaPlanetEntry["entryString"] = item2["entryString"]
            #dashaStrings.append(f'  AntarDasha: {item2["entryString"]} iid={idcnt}, open=False')
            #dashaCodeLines.append(f'tree.insert("", END, text = "{item2["entryString"]}", iid={idcnt}, open=False)')
            antardashaIdx = idcnt
            #dashaCodeLines.append(f'tree.move({antardashaIdx}, {dashaIdx}, {antar_pos}) ')
            paryantar_pos = 0
            idcnt = idcnt + 1
            antar_pos = antar_pos + 1
            #Adding Antardasha details
            antardasha_cnt = antardasha_cnt + 1
            antardashaPlanetName = antardashaPlanetEntry["name"]
            l_antardasha = {}
            l_antardasha["lord"] = antardashaPlanetName
            l_antardasha["dashaLord"] = mahadashaPlanetName
            l_antardasha["bhuktiNum"] = antardasha_cnt
            l_antardasha["startDate"] = str(item2["startDate"])
            l_antardasha["endDate"] = str(item2["endDate"])
            startage = (item2["startage"])
            endAge = (item2["endage"])
            l_dur = relativedelta((birthDate+endAge), (birthDate+startage))         
            l_antardasha["duration"] = f" {l_dur.years}yr {l_dur.months}m {l_dur.days}d"
            l_antardasha["startage"] = f" {startage.years}yr {startage.months}m {startage.days}d"
            l_antardasha["endage"] = f" {endAge.years}yr {endAge.months}m {endAge.days}d"  
            if(currentDate >= item2["startDate"]) and (currentDate < item2["endDate"]):
                data.charts["Dashas"]["Vimshottari"]["current"]["bhukti"] = l_antardasha.copy()["lord"]     
            data.charts["Dashas"]["Vimshottari"]["antardashas"][f'{mahadashaPlanetName}-{antardashaPlanetName}'] = l_antardasha.copy()

            #for Paryantardasha
            l_DashaStartDate = item2["startDate"]
            l_firstPlanet = item2["name"]
            l_wholeDuration = item2["endDate"] - item2["startDate"]
            res_paryantardasha = computeSubPeriods(l_DashaStartDate, l_firstPlanet, l_wholeDuration, "Antardasha", birthDate)
            paryantardasha_cnt = 0
            for item3 in res_paryantardasha:
                paryantardashaPlanetEntry["name"] = item3["name"]
                paryantardashaPlanetEntry["startDate"] = item3["startDate"]
                paryantardashaPlanetEntry["endDate"] = item3["endDate"]
                paryantardashaPlanetEntry["entryString"] = item3["entryString"]
                #dashaStrings.append(f'\tParyantarDasha: {item3["entryString"]} iid={idcnt}, open=False')
                #dashaCodeLines.append(f'tree.insert("", END, text = "{item3["entryString"]}", iid={idcnt}, open=False)')
                paryantardashaIdx = idcnt
                #dashaCodeLines.append(f'tree.move({paryantardashaIdx}, {antardashaIdx}, {paryantar_pos}) ')
                paryantar_pos = paryantar_pos + 1
                idcnt = idcnt + 1

                #Adding Paryantardasha details
                paryantardasha_cnt = paryantardasha_cnt + 1
                paryantardashaPlanetName = paryantardashaPlanetEntry["name"]
                l_paryantardasha = {}
                l_paryantardasha["lord"] = paryantardashaPlanetName
                l_paryantardasha["bhuktiLord"] = antardashaPlanetName
                l_paryantardasha["dashaLord"] = mahadashaPlanetName
                l_paryantardasha["pariNum"] = paryantardasha_cnt
                l_paryantardasha["startDate"] = str(item3["startDate"])
                l_paryantardasha["endDate"] = str(item3["endDate"])
                startage = (item3["startage"])
                endAge = (item3["endage"])
                l_dur = relativedelta((birthDate+endAge), (birthDate+startage))         
                l_paryantardasha["duration"] = f" {l_dur.years}yr {l_dur.months}m {l_dur.days}d"
                l_paryantardasha["startage"] = f" {startage.years}yr {startage.months}m {startage.days}d"
                l_paryantardasha["endage"] = f" {endAge.years}yr {endAge.months}m {endAge.days}d"       
                if(currentDate >= item3["startDate"]) and (currentDate < item3["endDate"]):
                    #print(f'''paryantar dasha {mahadashaPlanetName}-{antardashaPlanetName}-{paryantardashaPlanetName} ({currentDate} >= {item3["startDate"]}) and ({currentDate} > {item3["endDate"]})''')
                    data.charts["Dashas"]["Vimshottari"]["current"]["paryantardasha"] = l_paryantardasha.copy()["lord"]  
                data.charts["Dashas"]["Vimshottari"]["paryantardashas"][f'{mahadashaPlanetName}-{antardashaPlanetName}-{paryantardashaPlanetName}'] = l_paryantardasha.copy()

                antardashaPlanetEntry["paryantardasha"].append(item3.copy())

            mahadashaPlanetEntry["antardasha"].append(item2.copy())

        vimshottariDasha.append(mahadashaPlanetEntry.copy())
        #print(item["entryString"])
    return


#Takes the start date and time and starting planets and whole duration and computes a set
#the set of sub periods of planets in that duration like planet name, its start date, end date, duration and age
def computeSubPeriods(startdate,firstPlanet,wholeDuration, level, birthday):
    l_dashaStartDate = startdate
    l_dashaEndDate = l_dashaStartDate + wholeDuration
    l_120thpart = ((l_dashaEndDate - l_dashaStartDate)/120)
    l_focusplanet = firstPlanet
    l_planetStartDate = l_dashaStartDate
    l_SubPeriods = []
    l_planetEntry = {
                        "name": "",
                        "startDate": l_dashaStartDate,
                        "endDate": l_dashaEndDate,
                        "entryString": ""
                    }
    #print(f'{level}s under planet {firstPlanet}')
    for lplanetnum in range(9):   #for all nine planets
        l_planetDuration = (dashaVimshottariSkeleton[l_focusplanet]["duration"] * l_120thpart)
        l_planetEndDate = (l_planetStartDate + l_planetDuration)
        l_durDays = int(str(l_planetDuration).split("days")[0].strip())
        startAge = relativedelta(l_planetStartDate, birthday)
        endAge = relativedelta(l_planetEndDate, birthday)
        if((startAge.years <= 0) and (startAge.months <= 0) and (startAge.days <= 0)):
            startAge.years = 0
            startAge.months = 0
            startAge.days = 0
        if((endAge.years >= 0) and (endAge.months >= 0) and (endAge.days >= 0)):
            l_planetEntry["name"] = l_focusplanet
            l_planetEntry["startDate"] = l_planetStartDate
            l_planetEntry["endDate"] = l_planetEndDate
            l_planetEntry["duration"] = l_planetDuration
            l_planetEntry["startage"] = startAge
            l_planetEntry["endage"] = endAge
            l_planetEntry["entryString"] = f'{l_focusplanet} - ({str(l_planetStartDate)[:-4]}) to ({str(l_planetEndDate)[:-4]}) ({l_durDays} days) Age[({endAge.years}year {endAge.months}month {endAge.days}days)]'
            l_SubPeriods.append(l_planetEntry.copy())
            #print(f'{l_focusplanet} - ({l_planetStartDate}) to ({l_planetEndDate}) ({l_durDays} days) Age[({endAge.years}year {endAge.months}month{endAge.days}days)]')
        #setup needed for moving to next planet in chain
        l_planetStartDate = l_planetEndDate
        l_focusplanet = dashaVimshottariSkeleton[l_focusplanet]["next-dasha"]
    return(l_SubPeriods)

def Vimshottari(division, birthdata):
    moonlngsec = (((signnum(division["planets"]["Moon"]["sign"])-1) * 30 * 3600) + 
                  ((division["planets"]["Moon"]["pos"]["deg"]) * 3600) + 
                  ((division["planets"]["Moon"]["pos"]["min"]) * 60) + 
                  (division["planets"]["Moon"]["pos"]["sec"]))

    nakshatraLord = division["planets"]["Moon"]["nak-ruler"]

    bdaytime = datetime(year=birthdata["DOB"]["year"], 
                        month=birthdata["DOB"]["month"], 
                        day=birthdata["DOB"]["day"], 
                        hour=birthdata["TOB"]["hour"], 
                        minute=birthdata["TOB"]["min"], 
                        second=birthdata["TOB"]["sec"])

    computeVimshottariDasha(moonlngsec, nakshatraLord, bdaytime)
    #print(vimshottariDasha)
    return

if __name__ == "__main__":
    moonlngsec = (((signnum(data.lagna_moon["sign"])-1) * 30 * 3600) + 
                  ((data.lagna_moon["pos"]["deg"]) * 3600) + 
                  ((data.lagna_moon["pos"]["min"]) * 60) + 
                  (data.lagna_moon["pos"]["sec"]))

    nakshatraLord = data.lagna_moon["nak-ruler"]

    bdaytime = datetime(year=birthdata["DOB"]["year"], 
                        month=birthdata["DOB"]["month"], 
                        day=birthdata["DOB"]["day"], 
                        hour=birthdata["TOB"]["hour"], 
                        minute=birthdata["TOB"]["min"], 
                        second=birthdata["TOB"]["sec"])

    starttime = datetime(year=1989, month=11, day=11, hour=13, minute=8, second=48)
    #duration = relativedelta(years=dashaVimshottariSkeleton["Mars"]["duration"])
    #print(f'Duration of 7 years is {duration}')
    computeVimshottariDasha(moonlngsec, nakshatraLord, bdaytime)
    print(vimshottariDasha)
    #res = computeSubPeriods(starttime, "Mars", duration, "Antardasha", bdaytime)
    #for item in res:
        #print(item["entryString"])
    #print(debilitationSign_of_planet)




#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# mod_divisional.py -- module Divisional. All computations for Divisional chart [D2 to D60 chart - Total 15 divisional charts]
#
# Copyright (C) 2023 Shyam Bhat  <vicharavandana@gmail.com>
# Downloaded from "https://github.com/VicharaVandana/jyotishyam.git"
#
# This file is part of the "jyotishyam" Python library
# for computing Hindu jataka with sidereal lahiri ayanamsha technique 
# using swiss ephemeries
#
#Computed divisional charts with logic given in https://jothishi.com/divisional-charts/
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import parashara.mod_astrodata as data
import parashara.mod_constants as c
import parashara.mod_general as gen
from parashara.mod_lagna import *


#Below function computes Navamsa longitude from lagna longitude
def navamsa_from_long(sign, pos_deg, pos_min, pos_sec):
    #sign as 1:Aries, 2:Taurus ... 12:Pisces
    #pos_x is position of planet in sign in degree minute and seconds
    #this function computes navamsha position in sign and degree minute and seconds format only in tuple
    longi_sec = (((sign - 1) * 30 * 3600) +
                (pos_deg * 3600) + (pos_min * 60) + pos_sec)
    amsa = ((3 * 3600) + (20 * 60)) #one navamsa is 3 degree 20 minutes
    #check which navamsha does our planet falls in
    navSign = 1 + int(longi_sec/amsa) % 12
    longi_pending_sec = (longi_sec % amsa)
    longi_pending_sec_normalized = ((longi_pending_sec*30*3600)/amsa)   #pending is for 3deg20min so normalized is for 30deg
    navDeg = int(longi_pending_sec_normalized / 3600)
    longi_pending_sec_normalized = (longi_pending_sec_normalized % 3600)
    navMin = int(longi_pending_sec_normalized / 60)
    longi_pending_sec_normalized = (longi_pending_sec_normalized % 60)
    navSec = round(longi_pending_sec_normalized,2)
    longi_nav_sec = (((navSign - 1) * 30 * 3600) +
                (navDeg * 3600) + (navMin * 60) + navSec)
    return(longi_nav_sec, navSign, navDeg, navMin, navSec)

#Below function computes Dasamsa longitude from lagna longitude
def dasamsa_from_long(sign, pos_deg, pos_min, pos_sec):
    #sign as 1:Aries, 2:Taurus ... 12:Pisces
    #pos_x is position of planet in sign in degree minute and seconds
    #this function computes navamsha position in sign and degree minute and seconds format only in tuple
    longi_sec = ((pos_deg * 3600) + (pos_min * 60) + pos_sec)
    amsa = (3 * 3600) #one dasamsa is 3 degree 
    #check which dasamsa compartment does our planet falls in
    dasamsaCompartment = 1 + int(longi_sec/amsa) % 12
    #if lagna sign is even then dasamsa sign starts from 9th sign from lagna sign else from same sign
    if(sign % 2 == 0):  #even sign
        baseSign = gen.compute_nthsign(sign,9)
    else:
        baseSign = sign
    dasSign = gen.compute_nthsign(baseSign,dasamsaCompartment)
    longi_pending_sec = (longi_sec % amsa)
    longi_pending_sec_normalized = ((longi_pending_sec*30*3600)/amsa)   #pending is for 3deg20min so normalized is for 30deg
    dasDeg = int(longi_pending_sec_normalized / 3600)
    longi_pending_sec_normalized = (longi_pending_sec_normalized % 3600)
    dasMin = int(longi_pending_sec_normalized / 60)
    longi_pending_sec_normalized = (longi_pending_sec_normalized % 60)
    dasSec = round(longi_pending_sec_normalized,2)
    longi_nav_sec = (((dasSign - 1) * 30 * 3600) +
                (dasDeg * 3600) + (dasMin * 60) + dasSec)
    return(longi_nav_sec, dasSign, dasDeg, dasMin, dasSec)

#Below function computes Dasamsa longitude from lagna longitude
def hora_from_long(sign, pos_deg, pos_min, pos_sec):
    #sign as 1:Aries, 2:Taurus ... 12:Pisces
    #pos_x is position of planet in sign in degree minute and seconds
    #this function computes navamsha position in sign and degree minute and seconds format only in tuple
    longi_sec = (((sign - 1) * 30 * 3600) +
                (pos_deg * 3600) + (pos_min * 60) + pos_sec)
    amsa = (15 * 3600) #one hora amsa is 15 degree 
    horSign = 1 + int(longi_sec/amsa) % 12
    longi_pending_sec = (longi_sec % amsa)
    longi_pending_sec_normalized = ((longi_pending_sec*30*3600)/amsa)   #pending is for 3deg20min so normalized is for 30deg
    horDeg = int(longi_pending_sec_normalized / 3600)
    longi_pending_sec_normalized = (longi_pending_sec_normalized % 3600)
    horMin = int(longi_pending_sec_normalized / 60)
    longi_pending_sec_normalized = (longi_pending_sec_normalized % 60)
    horSec = round(longi_pending_sec_normalized,2)
    longi_hora_sec = (((horSign - 1) * 30 * 3600) +
                (horDeg * 3600) + (horMin * 60) + horSec)
    return(longi_hora_sec, horSign, horDeg, horMin, horSec)

#Below function computes Drekkana longitude from lagna longitude
def Drekkana_from_long(sign, pos_deg, pos_min, pos_sec):
    #sign as 1:Aries, 2:Taurus ... 12:Pisces
    #pos_x is position of planet in sign in degree minute and seconds
    #this function computes Drekkana position in sign and degree minute and seconds format only in tuple
    longi_sec = ((pos_deg * 3600) + (pos_min * 60) + pos_sec)
    amsa = (10 * 3600) #one Drekkana amsa is 10 degree 
    drekkanaCompartment = 1 + int(longi_sec/amsa) 
    if(drekkanaCompartment == 1):  #even sign
        DrekkanaSign = gen.compute_nthsign(sign,1)
    elif(drekkanaCompartment == 2):  #even sign
        DrekkanaSign = gen.compute_nthsign(sign,5)
    else:
        DrekkanaSign = gen.compute_nthsign(sign,9)
    longi_pending_sec = (longi_sec % amsa)
    longi_pending_sec_normalized = ((longi_pending_sec*30*3600)/amsa)   #pending is for 10degree so normalized is for 30deg
    DrekkanaDeg = int(longi_pending_sec_normalized / 3600)
    longi_pending_sec_normalized = (longi_pending_sec_normalized % 3600)
    DrekkanaMin = int(longi_pending_sec_normalized / 60)
    longi_pending_sec_normalized = (longi_pending_sec_normalized % 60)
    DrekkanaSec = round(longi_pending_sec_normalized,2)
    longi_Drekkana_sec = (((DrekkanaSign - 1) * 30 * 3600) +
                (DrekkanaDeg * 3600) + (DrekkanaMin * 60) + DrekkanaSec)
    return(longi_Drekkana_sec, DrekkanaSign, DrekkanaDeg, DrekkanaMin, DrekkanaSec)

#Below function computes Chaturtamsa longitude from lagna longitude
def Chaturtamsa_from_long(sign, pos_deg, pos_min, pos_sec):
    #sign as 1:Aries, 2:Taurus ... 12:Pisces
    #pos_x is position of planet in sign in degree minute and seconds
    #this function computes Chaturtamsa position in sign and degree minute and seconds format only in tuple
    longi_sec = ((pos_deg * 3600) + (pos_min * 60) + pos_sec)
    amsa = (30 * 3600) / 4 #one Chaturtamsa amsa is 30degree/4 
    ChaturtamsaCompartment = 1 + int(longi_sec/amsa) 
    if(ChaturtamsaCompartment == 1):  #even sign
        ChaturtamsaSign = gen.compute_nthsign(sign,1)
    elif(ChaturtamsaCompartment == 2):  #even sign
        ChaturtamsaSign = gen.compute_nthsign(sign,4)
    elif(ChaturtamsaCompartment == 3):  #even sign
        ChaturtamsaSign = gen.compute_nthsign(sign,7)
    else:
        ChaturtamsaSign = gen.compute_nthsign(sign,10)
    longi_pending_sec = (longi_sec % amsa)
    longi_pending_sec_normalized = ((longi_pending_sec*30*3600)/amsa)   #pending is for 10degree so normalized is for 30deg
    ChaturtamsaDeg = int(longi_pending_sec_normalized / 3600)
    longi_pending_sec_normalized = (longi_pending_sec_normalized % 3600)
    ChaturtamsaMin = int(longi_pending_sec_normalized / 60)
    longi_pending_sec_normalized = (longi_pending_sec_normalized % 60)
    ChaturtamsaSec = round(longi_pending_sec_normalized,2)
    longi_Chaturtamsa_sec = (((ChaturtamsaSign - 1) * 30 * 3600) +
                (ChaturtamsaDeg * 3600) + (ChaturtamsaMin * 60) + ChaturtamsaSec)
    return(longi_Chaturtamsa_sec, ChaturtamsaSign, ChaturtamsaDeg, ChaturtamsaMin, ChaturtamsaSec)

#Below function computes Saptamsa longitude from lagna longitude
def Saptamsa_from_long(sign, pos_deg, pos_min, pos_sec):
    #sign as 1:Aries, 2:Taurus ... 12:Pisces
    #pos_x is position of planet in sign in degree minute and seconds
    #this function computes Saptamsa position in sign and degree minute and seconds format only in tuple
    longi_sec = ((pos_deg * 3600) + (pos_min * 60) + pos_sec)
    amsa = (30 * 3600) / 7 #one Saptamsa amsa is 30degree/4 
    SaptamsaCompartment = 1 + int(longi_sec/amsa) 
    if(sign % 2 == 1):  #odd sign
        SaptamsaSign = gen.compute_nthsign(sign,SaptamsaCompartment)
    else:   #odd sign
        SaptamsaSign = gen.compute_nthsign(sign,SaptamsaCompartment + 6)
    longi_pending_sec = (longi_sec % amsa)
    longi_pending_sec_normalized = ((longi_pending_sec*30*3600)/amsa)   #pending is for 10degree so normalized is for 30deg
    SaptamsaDeg = int(longi_pending_sec_normalized / 3600)
    longi_pending_sec_normalized = (longi_pending_sec_normalized % 3600)
    SaptamsaMin = int(longi_pending_sec_normalized / 60)
    longi_pending_sec_normalized = (longi_pending_sec_normalized % 60)
    SaptamsaSec = round(longi_pending_sec_normalized,2)
    longi_Saptamsa_sec = (((SaptamsaSign - 1) * 30 * 3600) +
                (SaptamsaDeg * 3600) + (SaptamsaMin * 60) + SaptamsaSec)
    return(longi_Saptamsa_sec, SaptamsaSign, SaptamsaDeg, SaptamsaMin, SaptamsaSec)

#Below function computes Dwadasamsa longitude from lagna longitude
def Dwadasamsa_from_long(sign, pos_deg, pos_min, pos_sec):
    #sign as 1:Aries, 2:Taurus ... 12:Pisces
    #pos_x is position of planet in sign in degree minute and seconds
    #this function computes Dwadasamsa position in sign and degree minute and seconds format only in tuple
    longi_sec = ((pos_deg * 3600) + (pos_min * 60) + pos_sec)
    amsa = (30 * 3600) / 12 #one Dwadasamsa amsa is 30degree/12 
    DwadasamsaCompartment = 1 + int(longi_sec/amsa) % 12   
    DwadasamsaSign = gen.compute_nthsign(sign,DwadasamsaCompartment)
    
    longi_pending_sec = (longi_sec % amsa)
    longi_pending_sec_normalized = ((longi_pending_sec*30*3600)/amsa)   #pending is for 10degree so normalized is for 30deg
    DwadasamsaDeg = int(longi_pending_sec_normalized / 3600)
    longi_pending_sec_normalized = (longi_pending_sec_normalized % 3600)
    DwadasamsaMin = int(longi_pending_sec_normalized / 60)
    longi_pending_sec_normalized = (longi_pending_sec_normalized % 60)
    DwadasamsaSec = round(longi_pending_sec_normalized,2)
    longi_Dwadasamsa_sec = (((DwadasamsaSign - 1) * 30 * 3600) +
                (DwadasamsaDeg * 3600) + (DwadasamsaMin * 60) + DwadasamsaSec)
    return(longi_Dwadasamsa_sec, DwadasamsaSign, DwadasamsaDeg, DwadasamsaMin, DwadasamsaSec)

#Below function computes Shodasamsa longitude from lagna longitude
def Shodasamsa_from_long(sign, pos_deg, pos_min, pos_sec):
    #sign as 1:Aries, 2:Taurus ... 12:Pisces
    #pos_x is position of planet in sign in degree minute and seconds
    #this function computes Shodasamsa position in sign and degree minute and seconds format only in tuple
    longi_sec = ((pos_deg * 3600) + (pos_min * 60) + pos_sec)
    amsa = (30 * 3600) / 16 #one Shodasamsa amsa is 30degree/16
    ShodasamsaCompartment = 1 + int(longi_sec/amsa) 
    if(sign in [1,4,7,10]):  #Movable sign
        ShodasamsaSign = gen.compute_nthsign(1,ShodasamsaCompartment)
    elif(sign in [2,5,8,11]):  #Fixed sign
        ShodasamsaSign = gen.compute_nthsign(5,ShodasamsaCompartment)
    else:   #Dual sign
        ShodasamsaSign = gen.compute_nthsign(9,ShodasamsaCompartment)
    longi_pending_sec = (longi_sec % amsa)
    longi_pending_sec_normalized = ((longi_pending_sec*30*3600)/amsa)   #pending is for 10degree so normalized is for 30deg
    ShodasamsaDeg = int(longi_pending_sec_normalized / 3600)
    longi_pending_sec_normalized = (longi_pending_sec_normalized % 3600)
    ShodasamsaMin = int(longi_pending_sec_normalized / 60)
    longi_pending_sec_normalized = (longi_pending_sec_normalized % 60)
    ShodasamsaSec = round(longi_pending_sec_normalized,2)
    longi_Shodasamsa_sec = (((ShodasamsaSign - 1) * 30 * 3600) +
                (ShodasamsaDeg * 3600) + (ShodasamsaMin * 60) + ShodasamsaSec)
    return(longi_Shodasamsa_sec, ShodasamsaSign, ShodasamsaDeg, ShodasamsaMin, ShodasamsaSec)

#Below function computes Vimsamsa longitude from lagna longitude
def Vimsamsa_from_long(sign, pos_deg, pos_min, pos_sec):
    #sign as 1:Aries, 2:Taurus ... 12:Pisces
    #pos_x is position of planet in sign in degree minute and seconds
    #this function computes Vimsamsa position in sign and degree minute and seconds format only in tuple
    longi_sec = ((pos_deg * 3600) + (pos_min * 60) + pos_sec)
    amsa = (30 * 3600) / 20 #one Vimsamsa amsa is 30degree/30
    VimsamsaCompartment = 1 + int(longi_sec/amsa) 
    if(sign in [1,4,7,10]):  #Movable sign
        VimsamsaSign = gen.compute_nthsign(1,VimsamsaCompartment)
    elif(sign in [2,5,8,11]):  #Fixed sign
        VimsamsaSign = gen.compute_nthsign(9,VimsamsaCompartment)
    else:   #Dual sign
        VimsamsaSign = gen.compute_nthsign(5,VimsamsaCompartment)
    longi_pending_sec = (longi_sec % amsa)
    longi_pending_sec_normalized = ((longi_pending_sec*30*3600)/amsa)   #pending is for 10degree so normalized is for 30deg
    VimsamsaDeg = int(longi_pending_sec_normalized / 3600)
    longi_pending_sec_normalized = (longi_pending_sec_normalized % 3600)
    VimsamsaMin = int(longi_pending_sec_normalized / 60)
    longi_pending_sec_normalized = (longi_pending_sec_normalized % 60)
    VimsamsaSec = round(longi_pending_sec_normalized,2)
    longi_Vimsamsa_sec = (((VimsamsaSign - 1) * 30 * 3600) +
                (VimsamsaDeg * 3600) + (VimsamsaMin * 60) + VimsamsaSec)
    return(longi_Vimsamsa_sec, VimsamsaSign, VimsamsaDeg, VimsamsaMin, VimsamsaSec)

#Below function computes ChaturVimsamsa longitude from lagna longitude
def ChaturVimsamsa_from_long(sign, pos_deg, pos_min, pos_sec):
    #sign as 1:Aries, 2:Taurus ... 12:Pisces
    #pos_x is position of planet in sign in degree minute and seconds
    #this function computes ChaturVimsamsa position in sign and degree minute and seconds format only in tuple
    longi_sec = ((pos_deg * 3600) + (pos_min * 60) + pos_sec)
    amsa = (30 * 3600) / 24 #one ChaturVimsamsa amsa is 30degree/24
    ChaturVimsamsaCompartment = 1 + int(longi_sec/amsa) 
    if(sign % 2 == 0):  #Even sign
        ChaturVimsamsaSign = gen.compute_nthsign(4,ChaturVimsamsaCompartment)
    else:   #Odd sign
        ChaturVimsamsaSign = gen.compute_nthsign(5,ChaturVimsamsaCompartment)
    longi_pending_sec = (longi_sec % amsa)
    longi_pending_sec_normalized = ((longi_pending_sec*30*3600)/amsa)   #pending is for 10degree so normalized is for 30deg
    ChaturVimsamsaDeg = int(longi_pending_sec_normalized / 3600)
    longi_pending_sec_normalized = (longi_pending_sec_normalized % 3600)
    ChaturVimsamsaMin = int(longi_pending_sec_normalized / 60)
    longi_pending_sec_normalized = (longi_pending_sec_normalized % 60)
    ChaturVimsamsaSec = round(longi_pending_sec_normalized,2)
    longi_ChaturVimsamsa_sec = (((ChaturVimsamsaSign - 1) * 30 * 3600) +
                (ChaturVimsamsaDeg * 3600) + (ChaturVimsamsaMin * 60) + ChaturVimsamsaSec)
    return(longi_ChaturVimsamsa_sec, ChaturVimsamsaSign, ChaturVimsamsaDeg, ChaturVimsamsaMin, ChaturVimsamsaSec)

#Below function computes SaptaVimsamsa longitude from lagna longitude
def SaptaVimsamsa_from_long(sign, pos_deg, pos_min, pos_sec):
    #sign as 1:Aries, 2:Taurus ... 12:Pisces
    #pos_x is position of planet in sign in degree minute and seconds
    #this function computes SaptaVimsamsa position in sign and degree minute and seconds format only in tuple
    longi_sec = ((pos_deg * 3600) + (pos_min * 60) + pos_sec)
    amsa = (30 * 3600) / 27 #one SaptaVimsamsa amsa is 30degree/27
    SaptaVimsamsaCompartment = 1 + int(longi_sec/amsa) 
    if(sign in [1,5,9]):  #Fiery sign
        SaptaVimsamsaSign = gen.compute_nthsign(1,SaptaVimsamsaCompartment)
    elif(sign in [2,6,10]):  #Earthy sign
        SaptaVimsamsaSign = gen.compute_nthsign(4,SaptaVimsamsaCompartment)
    elif(sign in [3,7,11]):  #Airy sign
        SaptaVimsamsaSign = gen.compute_nthsign(7,SaptaVimsamsaCompartment)
    else:   #Watery sign
        SaptaVimsamsaSign = gen.compute_nthsign(10,SaptaVimsamsaCompartment)

    longi_pending_sec = (longi_sec % amsa)
    longi_pending_sec_normalized = ((longi_pending_sec*30*3600)/amsa)   #pending is for 10degree so normalized is for 30deg
    SaptaVimsamsaDeg = int(longi_pending_sec_normalized / 3600)
    longi_pending_sec_normalized = (longi_pending_sec_normalized % 3600)
    SaptaVimsamsaMin = int(longi_pending_sec_normalized / 60)
    longi_pending_sec_normalized = (longi_pending_sec_normalized % 60)
    SaptaVimsamsaSec = round(longi_pending_sec_normalized,2)
    longi_SaptaVimsamsa_sec = (((SaptaVimsamsaSign - 1) * 30 * 3600) +
                (SaptaVimsamsaDeg * 3600) + (SaptaVimsamsaMin * 60) + SaptaVimsamsaSec)
    return(longi_SaptaVimsamsa_sec, SaptaVimsamsaSign, SaptaVimsamsaDeg, SaptaVimsamsaMin, SaptaVimsamsaSec)

#Below function computes Trimsamsa longitude from lagna longitude
def Trimsamsa_from_long(sign, pos_deg, pos_min, pos_sec):
    #sign as 1:Aries, 2:Taurus ... 12:Pisces
    #pos_x is position of planet in sign in degree minute and seconds
    #this function computes Trimsamsa position in sign and degree minute and seconds format only in tuple
    longi_sec = ((pos_deg * 3600) + (pos_min * 60) + pos_sec)
    amsa = (30 * 3600) / 30 #one Trimsamsa amsa is 30degree/30
    #TrimsamsaCompartment = 1 + int(longi_sec/amsa) 
    if(sign % 2 == 0):  #Even Lagna in Rasi chart
        if(longi_sec <= 5 * 3600):
            TrimsamsaSign = 2
        elif(longi_sec <= 12 * 3600):
            TrimsamsaSign = 6
        elif(longi_sec <= 20 * 3600):
            TrimsamsaSign = 12
        elif(longi_sec <= 25 * 3600):
            TrimsamsaSign = 10
        else:
            TrimsamsaSign = 8
    else:   #Odd Lagna in Rasi chart
        if(longi_sec <= 5 * 3600):
            TrimsamsaSign = 1
        elif(longi_sec <= 10 * 3600):
            TrimsamsaSign = 11
        elif(longi_sec <= 18 * 3600):
            TrimsamsaSign = 9
        elif(longi_sec <= 25 * 3600):
            TrimsamsaSign = 3
        else:
            TrimsamsaSign = 7

    longi_pending_sec = (longi_sec % amsa)
    longi_pending_sec_normalized = ((longi_pending_sec*30*3600)/amsa)   #pending is for 10degree so normalized is for 30deg
    TrimsamsaDeg = int(longi_pending_sec_normalized / 3600)
    longi_pending_sec_normalized = (longi_pending_sec_normalized % 3600)
    TrimsamsaMin = int(longi_pending_sec_normalized / 60)
    longi_pending_sec_normalized = (longi_pending_sec_normalized % 60)
    TrimsamsaSec = round(longi_pending_sec_normalized,2)
    longi_Trimsamsa_sec = (((TrimsamsaSign - 1) * 30 * 3600) +
                (TrimsamsaDeg * 3600) + (TrimsamsaMin * 60) + TrimsamsaSec)
    return(longi_Trimsamsa_sec, TrimsamsaSign, TrimsamsaDeg, TrimsamsaMin, TrimsamsaSec)

#Below function computes Khavedamsa longitude from lagna longitude
def Khavedamsa_from_long(sign, pos_deg, pos_min, pos_sec):
    #sign as 1:Aries, 2:Taurus ... 12:Pisces
    #pos_x is position of planet in sign in degree minute and seconds
    #this function computes Khavedamsa position in sign and degree minute and seconds format only in tuple
    longi_sec = ((pos_deg * 3600) + (pos_min * 60) + pos_sec)
    amsa = (30 * 3600) / 40 #one Khavedamsa amsa is 30degree/40
    KhavedamsaCompartment = 1 + int(longi_sec/amsa) 
    if(sign % 2 == 0):  #Even sign
        KhavedamsaSign = gen.compute_nthsign(7,KhavedamsaCompartment)
    else:   #Odd sign
        KhavedamsaSign = gen.compute_nthsign(1,KhavedamsaCompartment)
    longi_pending_sec = (longi_sec % amsa)
    longi_pending_sec_normalized = ((longi_pending_sec*30*3600)/amsa)   #pending is for 10degree so normalized is for 30deg
    KhavedamsaDeg = int(longi_pending_sec_normalized / 3600)
    longi_pending_sec_normalized = (longi_pending_sec_normalized % 3600)
    KhavedamsaMin = int(longi_pending_sec_normalized / 60)
    longi_pending_sec_normalized = (longi_pending_sec_normalized % 60)
    KhavedamsaSec = round(longi_pending_sec_normalized,2)
    longi_Khavedamsa_sec = (((KhavedamsaSign - 1) * 30 * 3600) +
                (KhavedamsaDeg * 3600) + (KhavedamsaMin * 60) + KhavedamsaSec)
    return(longi_Khavedamsa_sec, KhavedamsaSign, KhavedamsaDeg, KhavedamsaMin, KhavedamsaSec)

#Below function computes Akshavedamsa longitude from lagna longitude
def Akshavedamsa_from_long(sign, pos_deg, pos_min, pos_sec):
    #sign as 1:Aries, 2:Taurus ... 12:Pisces
    #pos_x is position of planet in sign in degree minute and seconds
    #this function computes Akshavedamsa position in sign and degree minute and seconds format only in tuple
    longi_sec = ((pos_deg * 3600) + (pos_min * 60) + pos_sec)
    amsa = (30 * 3600) / 45 #one Akshavedamsa amsa is 30degree/45
    AkshavedamsaCompartment = 1 + int(longi_sec/amsa) 
    if(sign in [1,4,7,10]):  #Movable sign
        AkshavedamsaSign = gen.compute_nthsign(1,AkshavedamsaCompartment)
    elif(sign in [2,5,8,11]):  #Fixed sign
        AkshavedamsaSign = gen.compute_nthsign(5,AkshavedamsaCompartment)
    else:   #Dual sign
        AkshavedamsaSign = gen.compute_nthsign(9,AkshavedamsaCompartment)
    longi_pending_sec = (longi_sec % amsa)
    longi_pending_sec_normalized = ((longi_pending_sec*30*3600)/amsa)   #pending is for 10degree so normalized is for 30deg
    AkshavedamsaDeg = int(longi_pending_sec_normalized / 3600)
    longi_pending_sec_normalized = (longi_pending_sec_normalized % 3600)
    AkshavedamsaMin = int(longi_pending_sec_normalized / 60)
    longi_pending_sec_normalized = (longi_pending_sec_normalized % 60)
    AkshavedamsaSec = round(longi_pending_sec_normalized,2)
    longi_Akshavedamsa_sec = (((AkshavedamsaSign - 1) * 30 * 3600) +
                (AkshavedamsaDeg * 3600) + (AkshavedamsaMin * 60) + AkshavedamsaSec)
    return(longi_Akshavedamsa_sec, AkshavedamsaSign, AkshavedamsaDeg, AkshavedamsaMin, AkshavedamsaSec)

#Below function computes Shashtiamsa longitude from lagna longitude
def Shashtiamsa_from_long(sign, pos_deg, pos_min, pos_sec):
    #sign as 1:Aries, 2:Taurus ... 12:Pisces
    #pos_x is position of planet in sign in degree minute and seconds
    #this function computes Shashtiamsa position in sign and degree minute and seconds format only in tuple
    longi_sec = ((pos_deg * 3600) + (pos_min * 60) + pos_sec)
    amsa = (30 * 3600) / 60 #one Shashtiamsa amsa is 30degree/60 
    ShashtiamsaCompartment = 1 + int(longi_sec/amsa) % 12   
    ShashtiamsaSign = gen.compute_nthsign(sign,ShashtiamsaCompartment)
    
    longi_pending_sec = (longi_sec % amsa)
    longi_pending_sec_normalized = ((longi_pending_sec*30*3600)/amsa)   #pending is for 10degree so normalized is for 30deg
    ShashtiamsaDeg = int(longi_pending_sec_normalized / 3600)
    longi_pending_sec_normalized = (longi_pending_sec_normalized % 3600)
    ShashtiamsaMin = int(longi_pending_sec_normalized / 60)
    longi_pending_sec_normalized = (longi_pending_sec_normalized % 60)
    ShashtiamsaSec = round(longi_pending_sec_normalized,2)
    longi_Shashtiamsa_sec = (((ShashtiamsaSign - 1) * 30 * 3600) +
                (ShashtiamsaDeg * 3600) + (ShashtiamsaMin * 60) + ShashtiamsaSec)
    return(longi_Shashtiamsa_sec, ShashtiamsaSign, ShashtiamsaDeg, ShashtiamsaMin, ShashtiamsaSec)

def prepDivisionalchartElements(division):
    #prepare Ascendant
    division["ascendant"] = {   "name"        : "Ascendant",
                                "symbol"       : "Asc",
                                "pos"          : {"deg" : 0, "min" : 0, "sec" : 0, "dec_deg": 0.0}, #initioalized to zero
                                "nakshatra"    : "Ashwini" ,
                                "pada"         : 1,
                                "nak-ruler"    : "Ketu",
                                "nak-diety"    : "Ashwini kumaras",
                                "sign"         : "Aries",
                                "rashi"        : "Mesha",
                                "lagna-lord"   : "Mars",
                                "sign-tatva"   : c.FIRE,
                                "lagnesh-sign" : "Aries",
                                "lagnesh-rashi": "Mesha",
                                "lagnesh-disp" : "Mars",
                                "status"       : c.INIT
                                }.copy()
    division["planets"] = {}
    division["vargottamas"] = []    #only for non D1 divisional charts
    division["houses"] = []
    division["classifications"] = { "benefics"    : [],
                            "malefics"    : [],
                            "neutral"     : [],
                            "kendra"      : [],
                            "trikona"     : [],
                            "trik"        : [],
                            "upachaya"    : [],
                            "dharma"      : [],
                            "artha"       : [],
                            "kama"        : [],
                            "moksha"      : []
                          }.copy()
    division["planets"]["Sun"] = {  "name"         : "Sun",
                                    "symbol"       : "Su",
                                    "retro"        : 0,    #initialized retro as 0
                                    "pos"          : {"deg" : 0, "min" : 0, "sec" : 0, "dec_deg": 0.0}, #initioalized to zero
                                    "nakshatra"    : "Ashwini" ,
                                    "pada"         : 1,
                                    "nak-ruler"    : "Ketu",
                                    "nak-diety"    : "Ashwini kumaras",
                                    "sign"         : "Aries",
                                    "rashi"        : "Mesha",
                                    "dispositor"   : "Mars",
                                    "tattva"       : c.FIRE,
                                    "sign-tatva"   : c.FIRE,
                                    "house-rel"    : c.EXHALTED,
                                    "house-nature" : c.DHARMA,
                                    "planet-nature": c.PAAPAGRAHA,
                                    "gender"       : c.MALE,
                                    "category"     : c.DEVA,
                                    "house-num"    : 1,
                                    "friends"      : ["Moon", "Mars", "Jupiter"],
                                    "enemies"      : ["Venus", "Saturn", "Rahu", "Ketu"],
                                    "nuetral"      : ["Mercury"],
                                    "varna"        : c.KSHATRIYA,
                                    "guna"         : c.SATVA,
                                    "Aspects"      : {"planets":[], "houses":[], "signs":[]},
                                    "Aspected-by"  : [],
                                    "conjuncts"    : [],
                                    "status"       : c.INIT
                                    }.copy()
    division["planets"]["Moon"] = {"name"         : "Moon",
             "symbol"       : "Mo",
             "retro"        : 0,    #initialized retro as 0
             "pos"          : {"deg" : 0, "min" : 0, "sec" : 0, "dec_deg": 0.0}, #initioalized to zero
             "nakshatra"    : "Ashwini" ,
             "pada"         : 1,
             "nak-ruler"    : "Ketu",
             "nak-diety"    : "Ashwini kumaras",
             "sign"         : "Aries",
             "rashi"        : "Mesha",
             "dispositor"   : "Mars",
             "tattva"       : c.WATER,
             "sign-tatva"   : c.FIRE,
             "house-rel"    : c.FRIENDSIGN,
             "house-nature" : c.DHARMA,
             "planet-nature": c.PUNYAGRAHA,
             "gender"       : c.FEMALE,
             "category"     : c.DEVA,
             "house-num"    : 1,
             "friends"      : ["Sun", "Mercury"],
             "enemies"      : [],
             "nuetral"      : ["Mars", "Venus", "Jupiter", "Saturn", "Rahu", "Ketu"],
             "varna"        : c.VAISHYA,
             "guna"         : c.SATVA,
             "Aspects"      : {"planets":[], "houses":[], "signs":[]},
             "Aspected-by"  : [],
             "conjuncts"    : [],
             "status"       : c.INIT
            }.copy()
    division["planets"]["Mars"] = {"name"        : "Mars",
             "symbol"       : "Ma",
             "retro"        : 0,    #initialized retro as 0
             "pos"          : {"deg" : 0, "min" : 0, "sec" : 0, "dec_deg": 0.0}, #initioalized to zero
             "nakshatra"    : "Ashwini" ,
             "pada"         : 1,
             "nak-ruler"    : "Ketu",
             "nak-diety"    : "Ashwini kumaras",
             "sign"         : "Aries",
             "rashi"        : "Mesha",
             "dispositor"   : "Mars",
             "tattva"       : c.FIRE,
             "sign-tatva"   : c.FIRE,
             "house-rel"    : c.OWNSIGN,
             "house-nature" : c.DHARMA,
             "planet-nature": c.PAAPAGRAHA,
             "gender"       : c.MALE,
             "category"     : c.DEVA,
             "house-num"    : 1,
             "friends"      : ["Sun", "Moon", "Jupiter"],
             "enemies"      : ["Mercury"],
             "nuetral"      : ["Saturn", "Venus", "Rahu", "Ketu"],
             "varna"        : c.KSHATRIYA,
             "guna"         : c.TAMAS,
             "Aspects"      : {"planets":[], "houses":[], "signs":[]},
             "Aspected-by"  : [],
             "conjuncts"    : [],
             "status"       : c.INIT
            }.copy()
    division["planets"]["Mercury"] = {"name"     : "Mercury",
             "symbol"       : "Me",
             "retro"        : 0,    #initialized retro as 0
             "pos"          : {"deg" : 0, "min" : 0, "sec" : 0, "dec_deg": 0.0}, #initioalized to zero
             "nakshatra"    : "Ashwini" ,
             "pada"         : 1,
             "nak-ruler"    : "Ketu",
             "nak-diety"    : "Ashwini kumaras",
             "sign"         : "Aries",
             "rashi"        : "Mesha",
             "dispositor"   : "Mars",
             "tattva"       : c.EARTH,
             "sign-tatva"   : c.FIRE,
             "house-rel"    : c.NEUTRALSIGN,
             "house-nature" : c.DHARMA,
             "planet-nature": c.PAAPAGRAHA,
             "gender"       : c.NEUTER,
             "category"     : c.NEUTRAL,
             "house-num"    : 1,
             "friends"      : ["Sun", "Venus", "Rahu"],
             "enemies"      : ["Moon", "Ketu"],
             "nuetral"      : ["Saturn", "Mars", "Jupiter"],
             "varna"        : c.SHUDRA,
             "guna"         : c.RAJAS,
             "Aspects"      : {"planets":[], "houses":[], "signs":[]},
             "Aspected-by"  : [],
             "conjuncts"    : [],
             "status"       : c.INIT
            }.copy()
    division["planets"]["Jupiter"] = {"name"     : "Jupiter",
             "symbol"       : "Ju",
             "retro"        : 0,    #initialized retro as 0
             "pos"          : {"deg" : 0, "min" : 0, "sec" : 0, "dec_deg": 0.0}, #initioalized to zero
             "nakshatra"    : "Ashwini" ,
             "pada"         : 1,
             "nak-ruler"    : "Ketu",
             "nak-diety"    : "Ashwini kumaras",
             "sign"         : "Aries",
             "rashi"        : "Mesha",
             "dispositor"   : "Mars",
             "tattva"       : c.FIRE,
             "sign-tatva"   : c.FIRE,
             "house-rel"    : c.FRIENDSIGN,
             "house-nature" : c.DHARMA,
             "planet-nature": c.PUNYAGRAHA,
             "gender"       : c.MALE,
             "category"     : c.DEVA,
             "house-num"    : 1,
             "friends"      : ["Moon", "Mars", "Sun", "Ketu"],
             "enemies"      : ["Venus", "Mercury", "Rahu"],
             "nuetral"      : ["Saturn"],
             "varna"        : c.BRAHMIN,
             "guna"         : c.SATVA,
             "Aspects"      : {"planets":[], "houses":[], "signs":[]},
             "Aspected-by"  : [],
             "conjuncts"    : [],
             "status"       : c.INIT
            }.copy()
    division["planets"]["Venus"] = {"name"       : "Venus",
             "symbol"       : "Ve",
             "retro"        : 0,    #initialized retro as 0
             "pos"          : {"deg" : 0, "min" : 0, "sec" : 0, "dec_deg": 0.0}, #initioalized to zero
             "nakshatra"    : "Ashwini" ,
             "pada"         : 1,
             "nak-ruler"    : "Ketu",
             "nak-diety"    : "Ashwini kumaras",
             "sign"         : "Aries",
             "rashi"        : "Mesha",
             "dispositor"   : "Mars",
             "tattva"       : c.AIR,
             "sign-tatva"   : c.FIRE,
             "house-rel"    : c.NEUTRALSIGN,
             "house-nature" : c.DHARMA,
             "planet-nature": c.PUNYAGRAHA,
             "gender"       : c.FEMALE,
             "category"     : c.DANAVA,
             "house-num"    : 1,
             "friends"      : ["Saturn", "Mercury", "Rahu", "Ketu"],
             "enemies"      : ["Sun", "Moon"],
             "nuetral"      : ["Mars", "Jupiter"],
             "varna"        : c.BRAHMIN,
             "guna"         : c.RAJAS,
             "Aspects"      : {"planets":[], "houses":[], "signs":[]},
             "Aspected-by"  : [],
             "conjuncts"    : [],
             "status"       : c.INIT
            }.copy()
    division["planets"]["Saturn"] = {"name"      : "Saturn",
             "symbol"       : "Sa",
             "retro"        : 0,    #initialized retro as 0
             "pos"          : {"deg" : 0, "min" : 0, "sec" : 0, "dec_deg": 0.0}, #initioalized to zero
             "nakshatra"    : "Ashwini" ,
             "pada"         : 1,
             "nak-ruler"    : "Ketu",
             "nak-diety"    : "Ashwini kumaras",
             "sign"         : "Aries",
             "rashi"        : "Mesha",
             "dispositor"   : "Mars",
             "tattva"       : c.AIR,
             "sign-tatva"   : c.FIRE,
             "house-rel"    : c.DEBILITATED,
             "house-nature" : c.DHARMA,
             "planet-nature": c.PAAPAGRAHA,
             "gender"       : c.FEMALE,
             "category"     : c.DANAVA,
             "house-num"    : 1,
             "friends"      : ["Venus", "Mercury", "Rahu", "Ketu"],
             "enemies"      : ["Sun", "Moon", "Mars"],
             "nuetral"      : ["Jupiter"],
             "varna"        : c.SHUDRA,
             "guna"         : c.TAMAS,
             "Aspects"      : {"planets":[], "houses":[], "signs":[]},
             "Aspected-by"  : [],
             "conjuncts"    : [],
             "status"       : c.INIT
            }.copy()
    division["planets"]["Rahu"] = {"name"        : "Rahu",
             "symbol"       : "Ra",
             "retro"        : 1,    #initialized retro as 1
             "pos"          : {"deg" : 0, "min" : 0, "sec" : 0, "dec_deg": 0.0}, #initioalized to zero
             "nakshatra"    : "Ashwini" ,
             "pada"         : 1,
             "nak-ruler"    : "Ketu",
             "nak-diety"    : "Ashwini kumaras",
             "sign"         : "Aries",
             "rashi"        : "Mesha",
             "dispositor"   : "Mars",
             "tattva"       : c.AIR,
             "sign-tatva"   : c.FIRE,
             "house-rel"    : c.OWNSIGN,
             "house-nature" : c.DHARMA,
             "planet-nature": c.PAAPAGRAHA,
             "gender"       : c.MALE,
             "category"     : c.DANAVA,
             "house-num"    : 1,
             "friends"      : ["Venus", "Mercury", "Ketu", "Saturn"],
             "enemies"      : ["Sun", "Moon", "Mars"],
             "nuetral"      : ["Jupiter"],
             "varna"        : c.SHUDRA,
             "guna"         : c.TAMAS,
             "Aspects"      : {"planets":[], "houses":[], "signs":[]},
             "Aspected-by"  : [],
             "conjuncts"    : [],
             "status"       : c.INIT
            }.copy()
    division["planets"]["Ketu"] = {"name"        : "Ketu",
             "symbol"       : "Ke",
             "retro"        : 1,    #initialized retro as 1
             "pos"          : {"deg" : 0, "min" : 0, "sec" : 0, "dec_deg": 0.0}, #initioalized to zero
             "nakshatra"    : "Ashwini" ,
             "pada"         : 1,
             "nak-ruler"    : "Ketu",
             "nak-diety"    : "Ashwini kumaras",
             "sign"         : "Aries",
             "rashi"        : "Mesha",
             "dispositor"   : "Mars",
             "tattva"       : c.AIR,
             "sign-tatva"   : c.FIRE,
             "house-rel"    : c.OWNSIGN,
             "house-nature" : c.DHARMA,
             "planet-nature": c.PAAPAGRAHA,
             "gender"       : c.FEMALE,
             "category"     : c.DANAVA,
             "house-num"    : 1,
             "friends"      : ["Venus", "Mercury", "Rahu", "Saturn"],
             "enemies"      : ["Sun", "Moon", "Mars"],
             "nuetral"      : ["Jupiter"],
             "varna"        : c.SHUDRA,
             "guna"         : c.TAMAS,
             "Aspects"      : {"planets":[], "houses":[], "signs":[]},
             "Aspected-by"  : [],
             "conjuncts"    : [],
             "status"       : c.INIT
            }.copy()
    return

#Takes charts as input. For ascendant and all 9 planets, computes Divisional chart.
#input D1 of Charts and updates DX of charts - All the fields (x can be any division like 9 for navamsa, 10 for dasamsa etc)
def compute_Dx_4m_D1(charts, Dx):
    l_D1 = charts["D1"]
    l_Dx = charts[Dx]
    prepDivisionalchartElements(l_Dx)
    
    #for computing Navamsa Lagna
    #Gather inputs
    D1lagnaSign = (gen.signnum(l_D1["ascendant"]["sign"]))
    D1lagnaPosDeg = (l_D1["ascendant"]["pos"]["deg"])
    D1lagnaPosMin = (l_D1["ascendant"]["pos"]["min"])
    D1lagnaPosSec = (l_D1["ascendant"]["pos"]["sec"])
    #compute Navamsa of Ascendant
    if(Dx == "D9"):
        (div_fullLongi_sec, divSign, divDeg, divMin, divSec) = navamsa_from_long(D1lagnaSign, D1lagnaPosDeg, D1lagnaPosMin, D1lagnaPosSec)
    elif(Dx == "D10"):
        (div_fullLongi_sec, divSign, divDeg, divMin, divSec) = dasamsa_from_long(D1lagnaSign, D1lagnaPosDeg, D1lagnaPosMin, D1lagnaPosSec)
    elif(Dx == "D2"):
        (div_fullLongi_sec, divSign, divDeg, divMin, divSec) = hora_from_long(D1lagnaSign, D1lagnaPosDeg, D1lagnaPosMin, D1lagnaPosSec)
    elif(Dx == "D3"):
        (div_fullLongi_sec, divSign, divDeg, divMin, divSec) = Drekkana_from_long(D1lagnaSign, D1lagnaPosDeg, D1lagnaPosMin, D1lagnaPosSec)
    elif(Dx == "D4"):
        (div_fullLongi_sec, divSign, divDeg, divMin, divSec) = Chaturtamsa_from_long(D1lagnaSign, D1lagnaPosDeg, D1lagnaPosMin, D1lagnaPosSec)
    elif(Dx == "D7"):
        (div_fullLongi_sec, divSign, divDeg, divMin, divSec) = Saptamsa_from_long(D1lagnaSign, D1lagnaPosDeg, D1lagnaPosMin, D1lagnaPosSec)
    elif(Dx == "D12"):
        (div_fullLongi_sec, divSign, divDeg, divMin, divSec) = Dwadasamsa_from_long(D1lagnaSign, D1lagnaPosDeg, D1lagnaPosMin, D1lagnaPosSec)
    elif(Dx == "D16"):
        (div_fullLongi_sec, divSign, divDeg, divMin, divSec) = Shodasamsa_from_long(D1lagnaSign, D1lagnaPosDeg, D1lagnaPosMin, D1lagnaPosSec)
    elif(Dx == "D20"):
        (div_fullLongi_sec, divSign, divDeg, divMin, divSec) = Vimsamsa_from_long(D1lagnaSign, D1lagnaPosDeg, D1lagnaPosMin, D1lagnaPosSec)
    elif(Dx == "D24"):
        (div_fullLongi_sec, divSign, divDeg, divMin, divSec) = ChaturVimsamsa_from_long(D1lagnaSign, D1lagnaPosDeg, D1lagnaPosMin, D1lagnaPosSec)
    elif(Dx == "D27"):
        (div_fullLongi_sec, divSign, divDeg, divMin, divSec) = SaptaVimsamsa_from_long(D1lagnaSign, D1lagnaPosDeg, D1lagnaPosMin, D1lagnaPosSec)
    elif(Dx == "D30"):
        (div_fullLongi_sec, divSign, divDeg, divMin, divSec) = Trimsamsa_from_long(D1lagnaSign, D1lagnaPosDeg, D1lagnaPosMin, D1lagnaPosSec)
    elif(Dx == "D40"):
        (div_fullLongi_sec, divSign, divDeg, divMin, divSec) = Khavedamsa_from_long(D1lagnaSign, D1lagnaPosDeg, D1lagnaPosMin, D1lagnaPosSec)
    elif(Dx == "D45"):
        (div_fullLongi_sec, divSign, divDeg, divMin, divSec) = Akshavedamsa_from_long(D1lagnaSign, D1lagnaPosDeg, D1lagnaPosMin, D1lagnaPosSec)
    elif(Dx == "D60"):
        (div_fullLongi_sec, divSign, divDeg, divMin, divSec) = Shashtiamsa_from_long(D1lagnaSign, D1lagnaPosDeg, D1lagnaPosMin, D1lagnaPosSec)
    else:
        div_fullLongi_sec = (((D1lagnaSign - 1) * 30 * 3600) +
                (D1lagnaPosDeg * 3600) + (D1lagnaPosMin * 60) + D1lagnaPosSec)
        (divSign, divDeg, divMin, divSec) = (D1lagnaSign, D1lagnaPosDeg, D1lagnaPosMin, D1lagnaPosSec)

    lagna = divSign
    #update Navamsa lagna in charts
    l_Dx["ascendant"]["sign"] = gen.signs[divSign-1]
    l_Dx["ascendant"]["pos"]["deg"] = divDeg
    l_Dx["ascendant"]["pos"]["min"] = divMin
    l_Dx["ascendant"]["pos"]["sec"] = divSec
    l_Dx["ascendant"]["pos"]["dec_deg"] = ((divDeg*3600) + (divMin*60) + divSec)/3600
    l_Dx["ascendant"]["rashi"]      = gen.rashis[divSign-1]
    l_Dx["ascendant"]["lagna-lord"] = gen.signlords[divSign-1]
    l_Dx["ascendant"]["sign-tatva"] = gen.signtatvas[divSign-1]

    #update nakshatra related data for ascendant
    nak_pad = nakshatra_pada(div_fullLongi_sec / 3600)  #argument must be full longitude in degrees
    l_Dx["ascendant"]["nakshatra"] = nak_pad[0]
    l_Dx["ascendant"]["pada"] = nak_pad[1]
    l_Dx["ascendant"]["nak-ruler"] = gen.ruler_of_nakshatra[nak_pad[0]]
    l_Dx["ascendant"]["nak-diety"] = gen.diety_of_nakshatra[nak_pad[0]]

    #update for every planet
    for planetname in l_Dx["planets"]:
        #Gather inputs
        D1PlanetSign = (gen.signnum(l_D1["planets"][planetname]["sign"]))
        D1PlanetPosDeg = (l_D1["planets"][planetname]["pos"]["deg"])
        D1PlanetPosMin = (l_D1["planets"][planetname]["pos"]["min"])
        D1PlanetPosSec = (l_D1["planets"][planetname]["pos"]["sec"])
        #compute Navamsa of Ascendant
        #(div_fullLongi_sec, divSign, divDeg, divMin, divSec) = navamsa_from_long(D1PlanetSign, D1PlanetPosDeg, D1PlanetPosMin, D1PlanetPosSec)
        if(Dx == "D9"):
            (div_fullLongi_sec, divSign, divDeg, divMin, divSec) = navamsa_from_long(D1PlanetSign, D1PlanetPosDeg, D1PlanetPosMin, D1PlanetPosSec)
        elif(Dx == "D10"):
            (div_fullLongi_sec, divSign, divDeg, divMin, divSec) = dasamsa_from_long(D1PlanetSign, D1PlanetPosDeg, D1PlanetPosMin, D1PlanetPosSec)
        elif(Dx == "D2"):
            (div_fullLongi_sec, divSign, divDeg, divMin, divSec) = hora_from_long(D1PlanetSign, D1PlanetPosDeg, D1PlanetPosMin, D1PlanetPosSec)
        elif(Dx == "D3"):
            (div_fullLongi_sec, divSign, divDeg, divMin, divSec) = Drekkana_from_long(D1PlanetSign, D1PlanetPosDeg, D1PlanetPosMin, D1PlanetPosSec)
        elif(Dx == "D4"):
            (div_fullLongi_sec, divSign, divDeg, divMin, divSec) = Chaturtamsa_from_long(D1PlanetSign, D1PlanetPosDeg, D1PlanetPosMin, D1PlanetPosSec)
        elif(Dx == "D7"):
            (div_fullLongi_sec, divSign, divDeg, divMin, divSec) = Saptamsa_from_long(D1PlanetSign, D1PlanetPosDeg, D1PlanetPosMin, D1PlanetPosSec)
        elif(Dx == "D12"):
            (div_fullLongi_sec, divSign, divDeg, divMin, divSec) = Dwadasamsa_from_long(D1PlanetSign, D1PlanetPosDeg, D1PlanetPosMin, D1PlanetPosSec)
        elif(Dx == "D16"):
            (div_fullLongi_sec, divSign, divDeg, divMin, divSec) = Shodasamsa_from_long(D1PlanetSign, D1PlanetPosDeg, D1PlanetPosMin, D1PlanetPosSec)
        elif(Dx == "D20"):
            (div_fullLongi_sec, divSign, divDeg, divMin, divSec) = Vimsamsa_from_long(D1PlanetSign, D1PlanetPosDeg, D1PlanetPosMin, D1PlanetPosSec)
        elif(Dx == "D24"):
            (div_fullLongi_sec, divSign, divDeg, divMin, divSec) = ChaturVimsamsa_from_long(D1PlanetSign, D1PlanetPosDeg, D1PlanetPosMin, D1PlanetPosSec)
        elif(Dx == "D27"):
            (div_fullLongi_sec, divSign, divDeg, divMin, divSec) = SaptaVimsamsa_from_long(D1PlanetSign, D1PlanetPosDeg, D1PlanetPosMin, D1PlanetPosSec)
        elif(Dx == "D30"):
            (div_fullLongi_sec, divSign, divDeg, divMin, divSec) = Trimsamsa_from_long(D1PlanetSign, D1PlanetPosDeg, D1PlanetPosMin, D1PlanetPosSec)
        elif(Dx == "D40"):
            (div_fullLongi_sec, divSign, divDeg, divMin, divSec) = Khavedamsa_from_long(D1PlanetSign, D1PlanetPosDeg, D1PlanetPosMin, D1PlanetPosSec)
        elif(Dx == "D45"):
            (div_fullLongi_sec, divSign, divDeg, divMin, divSec) = Akshavedamsa_from_long(D1PlanetSign, D1PlanetPosDeg, D1PlanetPosMin, D1PlanetPosSec)
        elif(Dx == "D60"):
            (div_fullLongi_sec, divSign, divDeg, divMin, divSec) = Shashtiamsa_from_long(D1PlanetSign, D1PlanetPosDeg, D1PlanetPosMin, D1PlanetPosSec)
        else:
            div_fullLongi_sec = (((D1PlanetSign - 1) * 30 * 3600) +
                    (D1PlanetPosDeg * 3600) + (D1PlanetPosMin * 60) + D1PlanetPosSec)
            (divSign, divDeg, divMin, divSec) = (D1PlanetSign, D1PlanetPosDeg, D1PlanetPosMin, D1PlanetPosSec)

        #Updating the attributes
        if(D1PlanetSign == divSign):
            l_Dx["vargottamas"].append(planetname)

        l_Dx["planets"][planetname]["retro"] = l_D1["planets"][planetname]["retro"]
        currentsign = gen.signs[divSign-1]
        l_Dx["planets"][planetname]["sign"] = currentsign
        l_Dx["planets"][planetname]["pos"]["deg"] = divDeg
        l_Dx["planets"][planetname]["pos"]["min"] = divMin
        l_Dx["planets"][planetname]["pos"]["sec"] = divSec
        l_Dx["planets"][planetname]["pos"]["dec_deg"] = ((divDeg*3600) + (divMin*60) + divSec)/3600
        l_Dx["planets"][planetname]["rashi"]      = gen.rashis[divSign-1]
        l_Dx["planets"][planetname]["sign-tatva"] = gen.signtatvas[divSign-1]
        dispositor = gen.signlords[divSign-1]
        l_Dx["planets"][planetname]["dispositor"] = dispositor
        housenum = gen.housediff(lagna, divSign)
        l_Dx["planets"][planetname]["house-num"] = housenum
        if (housenum in [1,5,9]):
            l_Dx["planets"][planetname]["house-nature"] = c.DHARMA
        elif (housenum in [2,6,10]):
            l_Dx["planets"][planetname]["house-nature"] = c.ARTHA
        elif (housenum in [3,7,11]):
            l_Dx["planets"][planetname]["house-nature"] = c.KAMA
        else:
            l_Dx["planets"][planetname]["house-nature"] = c.MOKSHA
        #update nakshatra related data for ascendant
        nak_pad = nakshatra_pada(div_fullLongi_sec / 3600)  #argument must be full longitude in degrees
        l_Dx["planets"][planetname]["nakshatra"] = nak_pad[0]
        l_Dx["planets"][planetname]["pada"] = nak_pad[1]
        l_Dx["planets"][planetname]["nak-ruler"] = gen.ruler_of_nakshatra[nak_pad[0]]
        l_Dx["planets"][planetname]["nak-diety"] = gen.diety_of_nakshatra[nak_pad[0]]

        exhaltsign = gen.exhaltationSign_of_planet[planetname] 
        debilitsign = gen.debilitationSign_of_planet[planetname]
        friends = l_Dx["planets"][planetname]["friends"]
        enemies = l_Dx["planets"][planetname]["enemies"]
        neutral = l_Dx["planets"][planetname]["nuetral"]
        if(currentsign == exhaltsign):  #first check for exhaltation
            l_Dx["planets"][planetname]["house-rel"] = c.EXHALTED
        elif(currentsign == debilitsign): #next check for debilitated 
            l_Dx["planets"][planetname]["house-rel"] = c.DEBILITATED
        elif(planetname == dispositor): #next check for own sign 
            l_Dx["planets"][planetname]["house-rel"] = c.OWNSIGN
        elif(dispositor in friends): #next check for friend sign 
            l_Dx["planets"][planetname]["house-rel"] = c.FRIENDSIGN
        elif(dispositor in enemies): #next check for enemy sign 
            l_Dx["planets"][planetname]["house-rel"] = c.ENEMYSIGN
        elif(dispositor in neutral): #next check for neutral sign 
            l_Dx["planets"][planetname]["house-rel"] = c.NEUTRALSIGN
        else:
            l_Dx["planets"][planetname]["house-rel"] = "UNKNOWN"

    lagnesh = l_Dx["ascendant"]["lagna-lord"]  #get lagnesh
    l_Dx["ascendant"]["lagnesh-sign"]  = l_Dx["planets"][lagnesh]["sign"]  #check the sign of lagnesh
    l_Dx["ascendant"]["lagnesh-rashi"] = l_Dx["planets"][lagnesh]["rashi"] 
    l_Dx["ascendant"]["lagnesh-disp"]  = l_Dx["planets"][lagnesh]["dispositor"]
    
    #computing benefics, malefics and neutral planets for given lagna
    gen.compute_BenMalNeu4lagna(lagna,data.charts[Dx]["classifications"])

    gen.update_houses(data.charts[Dx])

    #computing aspects and conjunction planets
    gen.compute_aspects(data.charts[Dx])
    gen.compute_aspectedby(data.charts[Dx])  
    gen.compute_conjuncts(data.charts[Dx]) 

    #populating the classification part of divisional chart
    gen.populate_Natural_BeneficsMalefics(data.charts[Dx])  #Natural benefics and malefics
    gen.populate_kendraplanets(data.charts[Dx]) #kendra planets
    gen.populate_trikonaplanets(data.charts[Dx]) #trikona planets
    gen.populate_trikplanets(data.charts[Dx]) #trik planets
    gen.populate_upachayaplanets(data.charts[Dx]) #upachaya planets
    gen.populate_dharmaplanets(data.charts[Dx]) #dharma planets
    gen.populate_arthaplanets(data.charts[Dx]) #artha planets
    gen.populate_kamaplanets(data.charts[Dx]) #kama planets
    gen.populate_mokshaplanets(data.charts[Dx]) #moksha planets

    return



if __name__ == "__main__":
      print(f'Hora of Ketu is {Shodasamsa_from_long(3, 20, 33, 21.76)}')



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# mod_lagna.py -- module lagna. All computations for lagna chart [D1 chart]
#
# Copyright (C) 2022 Shyam Bhat  <vicharavandana@gmail.com>
# Downloaded from "https://github.com/VicharaVandana/jyotishyam.git"
#
# This file is part of the "jyotishyam" Python library
# for computing Hindu jataka with sidereal lahiri ayanamsha technique 
# using swiss ephemeries
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Use Swiss ephemeris to calculate planetery position of 9 vedic astrology planets 
and lagna(Ascendant)
"""

from collections import namedtuple as struct

import swisseph as swe

import parashara.mod_astrodata as data
import parashara.mod_constants as c
import parashara.mod_general as gen

#from support.mod_astrodata import birthdata as bd



Date = struct("Date", ["year", "month", "day"])
Place = struct("Place", ["latitude", "longitude", "timezone"])

sidereal_year = 365.256360417   # From 


# namah suryaya chandraya mangalaya ... rahuve ketuve namah
swe.KETU = swe.PLUTO  # I've mapped Pluto to Ketu
planet_list = [swe.SUN, swe.MOON, swe.MARS, swe.MERCURY, swe.JUPITER,
               swe.VENUS, swe.SATURN, swe.MEAN_NODE, # Rahu = MEAN_NODE
               swe.KETU]

set_ayanamsa_mode = lambda: swe.set_sid_mode(swe.SIDM_LAHIRI)           #Vedic astrology uses Lahiri ayanamsa 
reset_ayanamsa_mode = lambda: swe.set_sid_mode(swe.SIDM_FAGAN_BRADLEY)  #Default is FAGAN_BRADLEY ayanamsa in swiss ephemeries

################################# FUNCTIONS #############################
def get_planet_name(planet):
  names = { swe.SUN: "Sun", swe.MOON: "Moon", swe.MARS: "Mars",
            swe.MERCURY: "Mercury", swe.JUPITER: "Jupiter", swe.VENUS: "Venus",
            swe.SATURN: "Saturn", swe.MEAN_NODE: "Rahu", swe.KETU: "Ketu"}
  return names[planet]

def get_planet_symbol(planet):
  symbols = { swe.SUN: "Su", swe.MOON: "Mo", swe.MARS: "Ma",
              swe.MERCURY: "Me", swe.JUPITER: "Ju", swe.VENUS: "Ve",
              swe.SATURN: "Sa", swe.MEAN_NODE: "Ra", swe.KETU: "Ke"}
  return symbols[planet]

# Convert 23d 30' 30" to 23.508333 degrees
from_dms = lambda degs, mins, secs: degs + mins/60 + secs/3600

# the inverse
def to_dms_prec(deg):
  d = int(deg)
  mins = (deg - d) * 60
  m = int(mins)
  s = round((mins - m) * 60, 6)
  return [d, m, s]

def to_dms(deg):
  d, m, s = to_dms_prec(deg)
  return [d, m, int(s)]

# Make angle lie between [-180, 180) instead of [0, 360)
norm180 = lambda angle: (angle - 360) if angle >= 180 else angle;

# Make angle lie between [0, 360)
norm360 = lambda angle: angle % 360

# Ketu is always 180° after Rahu, so same coordinates but different constellations
# i.e if Rahu is in Pisces, Ketu is in Virgo etc
ketu = lambda rahu: (rahu + 180) % 360

# Julian Day number as on (year, month, day) at 00:00 UTC
gregorian_to_jd = lambda date: swe.julday(date.year, date.month, date.day, 0.0)
jd_to_gregorian = lambda jd: swe.revjul(jd, swe.GREG_CAL)   # returns (y, m, d, h, min, s)

get_nakshatra_name = [  "Ashwini", "Bharani", "Kritika", 
                        "Rohini", "Mrigashira", "Ardra", 
                        "Punarvasu", "Pushya", "Ashlesha", 
                        "Magha", "Purva Phalguni", "Uttara Phalguni", 
                        "Hasta", "Chitra", "Swati", 
                        "Vishaka", "Anurada", "Jyeshta", 
                        "Mula", "Purva Ashadha", "Uttara Ashadha", 
                        "Shravana", "Dhanishta", "Shatabhishak", 
                        "Purva Bhadrapada", "Uttara Bhadrapada", "Revati"]

def nakshatra_pada(longitude):
  """Gives nakshatra (0..26) and paada (1..4) in which given longitude lies"""
  # 27 nakshatras span 360°
  one_star = (360 / 27)  # = 13°20'
  # Each nakshatra has 4 padas, so 27 x 4 = 108 padas in 360°
  one_pada = (360 / 108) # = 3°20'
  quotient = int(longitude / one_star)
  reminder = (longitude - quotient * one_star)
  pada = int(reminder / one_pada)
  # convert 0..26 to 1..27 and 0..3 to 1..4
  return [get_nakshatra_name[quotient], 1 + pada, quotient]

def sidereal_longitude(jd, planet):
  """Computes nirayana (sidereal) longitude of given planet on jd"""
  set_ayanamsa_mode()
  (longi,myflags) = swe.calc_ut(jd, planet, flags = swe.FLG_SWIEPH | swe.FLG_SIDEREAL)
  reset_ayanamsa_mode()
  return norm360(longi[0]) # degrees

def Is_Retrograde(jd, planet):
  """Checks if given planet is in retrograde motion on jd"""
  set_ayanamsa_mode()
  (longi,myflags) = swe.calc_ut(jd, planet, flags = swe.FLG_SWIEPH | swe.FLG_SPEED | swe.FLG_SIDEREAL)
  reset_ayanamsa_mode()
  return (longi[3] < 0) # if speed is negative then its in retro




def update_ascendant(jd, place):
  """Lagna (=ascendant) calculation at any given time & place
     It also updates most of lagna elements data, 
     except lagnesh_sign, lagnesh rashi and lagnesh dispositor"""
  lat, lon, tz = place
  jd_utc = jd - (tz / 24.)
  set_ayanamsa_mode() # needed for swe.houses_ex()
  # returns two arrays, cusps and ascmc, where ascmc[0] = Ascendant
  nirayana_lagna = swe.houses_ex(jd_utc, lat, lon, flags = swe.FLG_SIDEREAL)[1][0]
  # 12 zodiac signs span 360°, so each one takes 30°
  # 0 = Mesha, 1 = Vrishabha, ..., 11 = Meena
  constellation = int(nirayana_lagna / 30)
  coordinates = to_dms(nirayana_lagna % 30)
  reset_ayanamsa_mode()
  #Updating the data from computed values
  #update position of ascendant
  data.lagna_ascendant["pos"]["deg"] = coordinates[0]
  data.lagna_ascendant["pos"]["min"] = coordinates[1]
  data.lagna_ascendant["pos"]["sec"] = coordinates[2]
  data.lagna_ascendant["pos"]["dec_deg"] = (nirayana_lagna % 30)

  #update nakshatra related data for ascendant
  nak_pad = nakshatra_pada(nirayana_lagna)
  data.lagna_ascendant["nakshatra"] = nak_pad[0]
  data.lagna_ascendant["nakshatra-num"] = nak_pad[2]
  data.lagna_ascendant["pada"] = nak_pad[1]
  data.lagna_ascendant["nak-ruler"] = gen.ruler_of_nakshatra[nak_pad[0]]
  data.lagna_ascendant["nak-diety"] = gen.diety_of_nakshatra[nak_pad[0]]

  #update sign related data for ascendant
  data.lagna_ascendant["sign"]       = gen.signs[constellation]
  data.lagna_ascendant["rashi"]      = gen.rashis[constellation]
  data.lagna_ascendant["lagna-lord"] = gen.signlords[constellation]
  data.lagna_ascendant["sign-tatva"] = gen.signtatvas[constellation]

  #updating Status of Ascendant
  data.lagna_ascendant["status"] = c.PARTIAL

  return (1 + constellation)

def update_planetaryData(jd, place):
  """Computes instantaneous planetary positions
     (i.e., which celestial object lies in which constellation)
     Also gives the nakshatra-pada division
     And updates the birth chart data for all the planets except those dependant on ascendant
   """
  jd_ut = jd - place.timezone / 24.

  for planet in planet_list:
    if planet != swe.KETU:
      nirayana_long = sidereal_longitude(jd_ut, planet)
      retro = Is_Retrograde(jd_ut, planet)
    else: # Ketu
      #nirayana_long = ketu(sidereal_longitude(jd_ut, swe.RAHU))
      nirayana_long = ketu(sidereal_longitude(jd_ut, swe.MEAN_NODE))
      retro = True  #ketu is always in retrograde

    # 12 zodiac signs span 360°, so each one takes 30°
    # 0 = Mesha, 1 = Vrishabha, ..., 11 = Meena
    constellation = int(nirayana_long / 30)
    coordinates = to_dms(nirayana_long % 30)
    
    #Update the data properly for the planet
    db_planet = data.lagna_planets[get_planet_name(planet)] #get the proper planet container
    db_planet["retro"] = retro  #retrograde property

    #update position of the planet
    db_planet["pos"]["deg"] = coordinates[0]
    db_planet["pos"]["min"] = coordinates[1]
    db_planet["pos"]["sec"] = coordinates[2]
    db_planet["pos"]["dec_deg"] = (nirayana_long % 30)

    #update nakshatra related data for the planet
    nak_pad = nakshatra_pada(nirayana_long)
    db_planet["nakshatra"] = nak_pad[0]
    db_planet["pada"] = nak_pad[1]
    db_planet["nak-ruler"] = gen.ruler_of_nakshatra[nak_pad[0]]
    db_planet["nak-diety"] = gen.diety_of_nakshatra[nak_pad[0]]

    #update sign related data for the planet
    currentsign = gen.signs[constellation]
    db_planet["sign"]       = currentsign
    db_planet["rashi"]      = gen.rashis[constellation]
    dispositor = gen.signlords[constellation]
    db_planet["dispositor"] = dispositor
    db_planet["sign-tatva"] = gen.signtatvas[constellation]

    #Compute the house relation for the planet
    
    exhaltsign = gen.exhaltationSign_of_planet[db_planet["name"]] 
    debilitsign = gen.debilitationSign_of_planet[db_planet["name"]]
    friends = db_planet["friends"]
    enemies = db_planet["enemies"]
    neutral = db_planet["nuetral"]  

    if(currentsign == exhaltsign):  #first check for exhaltation
      db_planet["house-rel"] = c.EXHALTED
    elif(currentsign == debilitsign): #next check for debilitated 
      db_planet["house-rel"] = c.DEBILITATED
    elif(db_planet["name"] == dispositor): #next check for own sign 
      db_planet["house-rel"] = c.OWNSIGN
    elif(dispositor in friends): #next check for friend sign 
      db_planet["house-rel"] = c.FRIENDSIGN
    elif(dispositor in enemies): #next check for enemy sign 
      db_planet["house-rel"] = c.ENEMYSIGN
    elif(dispositor in neutral): #next check for neutral sign 
      db_planet["house-rel"] = c.NEUTRALSIGN
    else:
      db_planet["house-rel"] = "UNKNOWN"
    
    #Planet with dispositor relation

    
    #updating Status of planet
    db_planet["status"] = c.PARTIAL

  return



def compute_lagnaChart_custom(UserBirthData):
  birthday_julien = swe.julday( UserBirthData["DOB"]["year"],  #birth year
                                UserBirthData["DOB"]["month"],  #birth month
                                UserBirthData["DOB"]["day"],  #birth day
                                ((UserBirthData["TOB"]["hour"])+ (UserBirthData["TOB"]["min"])/60. + (UserBirthData["TOB"]["sec"])/3600),  #birth time in float
                              )   #yyyy,mm,dd,time_24hr_format(hh + mm/60 + ss/3600)
  
  birth_place = Place( UserBirthData["POB"]["lat"], #lattitude
                       UserBirthData["POB"]["lon"], #longitude
                       UserBirthData["POB"]["timezone"]  #Timezone
                      )
  lagna = update_ascendant(birthday_julien, birth_place)  #Compute ascendant related data
  update_planetaryData(birthday_julien, birth_place)  #Compute navagraha related data

  #update miscdata like name maasa vaara tithi etc
  data.charts["user_details"]["name"] = UserBirthData["name"]
  gen.update_miscdata(birthday_julien, birth_place, data.charts["user_details"])

  #computing benefics, malefics and neutral planets for given lagna
  gen.compute_BenMalNeu4lagna(lagna,data.D1["classifications"])

  #computing lagnesh related data for ascendant - not updated by update_ascendant()
  lagnesh = data.lagna_ascendant["lagna-lord"]  #get lagnesh
  data.lagna_ascendant["lagnesh-sign"]  = data.lagna_planets[lagnesh]["sign"]  #check the sign of lagnesh
  data.lagna_ascendant["lagnesh-rashi"] = data.lagna_planets[lagnesh]["rashi"] 
  data.lagna_ascendant["lagnesh-disp"]  = data.lagna_planets[lagnesh]["dispositor"] 
  #updating Status of Ascendant
  data.lagna_ascendant["status"] = c.COMPUTED

  #computing house related data for planets - not updated by update_planetaryData()
  for planetname in data.lagna_planets:
    planet = data.lagna_planets[planetname]
    planet["house-num"] = gen.housediff(lagna, gen.signnum(planet["sign"]))
    planet["chalit-house-num"] = gen.housediffAsc(planet["house-num"], 
                                                  planet["pos"]["dec_deg"],
                                                  data.lagna_ascendant["pos"]["dec_deg"])
    #updating Status of the planet
    planet["status"] = c.COMPUTED

  gen.update_houses(data.D1)

  #computing aspects and conjunction planets
  gen.compute_aspects(data.D1)
  gen.compute_aspectedby(data.D1)  
  gen.compute_conjuncts(data.D1) 

  #populating the classification part of divisional chart
  gen.populate_Natural_BeneficsMalefics(data.D1)  #Natural benefics and malefics
  gen.populate_kendraplanets(data.D1) #kendra planets
  gen.populate_trikonaplanets(data.D1) #trikona planets
  gen.populate_trikplanets(data.D1) #trik planets
  gen.populate_upachayaplanets(data.D1) #upachaya planets
  gen.populate_dharmaplanets(data.D1) #dharma planets
  gen.populate_arthaplanets(data.D1) #artha planets
  gen.populate_kamaplanets(data.D1) #kama planets
  gen.populate_mokshaplanets(data.D1) #moksha planets

  data.charts["user_details"]["nakshatra"] = data.lagna_moon["nakshatra"]
  data.charts["user_details"]["rashi"] = data.lagna_moon["sign"]
  data.charts["user_details"]["birthdetails"] = UserBirthData.copy()

  data.isAstroDataComputed = True
  return

if __name__ == "__main__":
    #compute_lagnaChart()
    print("GOOD")






#The AshtakaVarga calculation is made using method provided in below webpage:
# Webpage: https://www.thevedichoroscope.com/asthvarga-lessons/asthvarga-lessons-1/

import parashara.mod_astrodata as data
import parashara.mod_general as gen

#Bhinna ashtaka varga mappings 
BAV_BinduMatrix = {
    "Sun": { "Sun" : [1,2,4,7,8,9,10,11],   #Sun is benefic from suns 1st, 2nd, 4th, ... 11th houses
            "Moon" : [3,6,10,11],   #Sun is benefic from Moon in following positions 3,6,10,11.
            "Mars" : [1,2,4,7,8,9,10,11],
            "Mercury" : [3,5,6,9,10,11,12],
            "Jupiter" : [5,6,9,11],
            "Venus" : [6,7,12],
            "Saturn" : [1,2,4,7,8,9,10,11],
            "Ascendant" : [3,4,6,10,11,12]
            },
    "Moon": { "Sun" : [3,6,7,8,10,11],   
                "Moon" : [1,3,6,7,10,11],   
                "Mars" : [2,3,5,6,9,10,11],
                "Mercury" : [1,3,4,5,7,8,10,11],
                "Jupiter" : [1,4,7,8,10,11,12],
                "Venus" : [3,4,5,7,9,10,11],
                "Saturn" : [3,5,6,11],
                "Ascendant" : [3,6,10,11]
            },
    "Mars": { "Sun" : [3,5,6,10,11],   
                "Moon" : [3,6,11],   
                "Mars" : [1,2,4,7,8,10,11],
                "Mercury" : [3,5,6,11],
                "Jupiter" : [6,10,11,12],
                "Venus" : [6,8,11,12],
                "Saturn" : [1,7,8,9,10,11],
                "Ascendant" : [1,3,6,10,11]
            },
    "Mercury": { "Sun" : [5,6,9,11,12],   
                "Moon" : [2,4,6,8,10,11],   
                "Mars" : [1,2,4,7,8,9,10,12],
                "Mercury" : [1,3,5,6,9,10,11,12],
                "Jupiter" : [6,8,11,12],
                "Venus" : [1,2,3,4,5,8,9,11],
                "Saturn" : [1,2,4,7,8,9,10,12],
                "Ascendant" : [1,2,4,6,8,10,11]
            },
    "Jupiter": { "Sun" : [1,2,3,4,7,8,9,10,11],   
                "Moon" : [2,5,7,9,11],   
                "Mars" : [1,2,4,7,8,10,11],
                "Mercury" : [1,2,4,5,6,9,10,11],
                "Jupiter" : [1,2,3,4,7,8,10,11],
                "Venus" : [2,5,6,9,10,11],
                "Saturn" : [3,5,6,12],
                "Ascendant" : [1,2,4,5,6,7,9,10,11]
            },
    "Venus": { "Sun" : [8,11,12],   
                "Moon" : [1,2,3,4,5,8,9,11,12],   
                "Mars" : [3,4,6,9,11,12],
                "Mercury" : [3,5,6,9,11],
                "Jupiter" : [5,8,9,10,11],
                "Venus" : [1,2,3,4,5,8,9,10,11],
                "Saturn" : [3,4,5,8,9,10,11],
                "Ascendant" : [1,2,3,4,5,8,9,11]
            },
    "Saturn": { "Sun" : [1,2,4,7,8,10,11],   
                "Moon" : [3,6,11],   
                "Mars" : [3,5,6,10,11,12],
                "Mercury" : [6,8,9,10,11,12],
                "Jupiter" : [5,6,11,12],
                "Venus" : [6,11,12],
                "Saturn" : [3,5,6,11],
                "Ascendant" : [1,3,4,10,11]
            }
        }


def compute_AshtakaVargas():
    BhinnaAshtakaVargaPoints = {"Sun" : [0,0,0,0,0,0,0,0,0,0,0,0],
                            "Moon" : [0,0,0,0,0,0,0,0,0,0,0,0],
                            "Mars" : [0,0,0,0,0,0,0,0,0,0,0,0],
                            "Mercury" : [0,0,0,0,0,0,0,0,0,0,0,0],
                            "Jupiter" : [0,0,0,0,0,0,0,0,0,0,0,0],
                            "Venus" : [0,0,0,0,0,0,0,0,0,0,0,0],
                            "Saturn" : [0,0,0,0,0,0,0,0,0,0,0,0],
                            "Total" : [0,0,0,0,0,0,0,0,0,0,0,0]
                            }
    
    for planet in BAV_BinduMatrix:
        for refPlanet in BAV_BinduMatrix[planet]:
            planet_BAV_points = BhinnaAshtakaVargaPoints[planet]
            if (refPlanet == "Ascendant"):
                refplanet_housenum = 1
            else:
                refplanet_housenum = data.charts["D1"]["planets"][refPlanet]["house-num"]
            for nth in BAV_BinduMatrix[planet][refPlanet]:
                index = gen.compute_nthsign(refplanet_housenum, nth) - 1
                planet_BAV_points[index] = planet_BAV_points[index] + 1
                #print(f'''planet[{planet}] - refplanet[{refPlanet} in {nth}th house {refplanet_housenum}] - adding 1 point to house {index+1} \nto make the new matrix is {planet_BAV_points}''')
    for hno in range(12):
        BhinnaAshtakaVargaPoints["Total"][hno] = (  BhinnaAshtakaVargaPoints["Sun"][hno] +
                                                    BhinnaAshtakaVargaPoints["Moon"][hno] +
                                                    BhinnaAshtakaVargaPoints["Mars"][hno] +
                                                    BhinnaAshtakaVargaPoints["Mercury"][hno] + 
                                                    BhinnaAshtakaVargaPoints["Jupiter"][hno] +
                                                    BhinnaAshtakaVargaPoints["Venus"][hno] +
                                                    BhinnaAshtakaVargaPoints["Saturn"][hno])
    data.charts["AshtakaVarga"] = BhinnaAshtakaVargaPoints.copy()
    return


if __name__ == '__main__':
    compute_AshtakaVargas()
    print(BhinnaAshtakaVargaPoints["Sun"])
        